#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#define PWM_VALUE 63
//int gamma_table[PWM_VALUE+1] = {
//    0, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 9, 10,
//    11, 12, 13, 15, 17, 19, 21, 23, 26, 29, 32, 36, 40, 44, 49, 55,
//    61, 68, 76, 85, 94, 105, 117, 131, 146, 162, 181, 202, 225, 250,
//    279, 311, 346, 386, 430, 479, 534, 595, 663, 739, 824, 918, 1023
//};

int gamma_table[PWM_VALUE + 1] = {
  0, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 7, 8, 9, 10,
  11, 12, 13, 15, 17, 19, 21, 23, 26, 29, 32, 36, 40, 44, 49, 55,
  61, 68, 76, 85, 94, 105, 117, 131, 146, 162, 181, 202, 225, 250,
  279, 311, 346, 386, 430, 479, 534, 595, 663, 739, 824, 918, 1023
};



// RGB FET
#define redPIN    12
#define greenPIN  15
#define bluePIN   13


// W FET
#define w1PIN     14
#define w2PIN     4

// onbaord green LED D1
#define LEDPIN    5
// onbaord red LED D2
#define LED2PIN   1

// note
// TX GPIO2 @Serial1 (Serial ONE)
// RX GPIO3 @Serial


#define LEDoff digitalWrite(LEDPIN,HIGH)
#define LEDon digitalWrite(LEDPIN,LOW)

#define LED2off digitalWrite(LED2PIN,HIGH)
#define LED2on digitalWrite(LED2PIN,LOW)


// Update these with values suitable for your network.

const char* ssid = "wifi1337";
const char* password = "1337CryptoMaster";
const char* mqtt_server = "public.example.com";
#define MQTTCLIENTNAME "LED-Strip"
#define MQTTUSERNAME "username"
#define MQTTPASSWORD "password"

WiFiClient espClient;
PubSubClient client(espClient);
long lastMsg = 0;
char msg[50];
int value = 0;

bool compareString(char* a, char* b) {
  if (strcmp(a, b) == 0)
    return true;
  else
    return false;
}

void setup() {
  pinMode(LEDPIN, OUTPUT);
  pinMode(LED2PIN, OUTPUT);

  pinMode(redPIN, OUTPUT);
  pinMode(greenPIN, OUTPUT);
  pinMode(bluePIN, OUTPUT);
  pinMode(w1PIN, OUTPUT);
  pinMode(w2PIN, OUTPUT);


  Serial.begin(115200);
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void setup_wifi() {

  delay(10);
  // We start by connecting to a WiFi network
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
}

void callback(char* topic, byte* payloadChar, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  //  if (compareString(topic, "inTopic"))
  //    Serial.print("Geklappt!");
  for (int i = 0; i < length; i++) {
    Serial.print((char)payloadChar[i]);
  }
  Serial.println();

  String payload((char*)payloadChar);

  if (compareString(topic, "/openHAB/RGB_2/Color")) {
    int c1 = payload.indexOf(';');
    int c2 = payload.indexOf(';', c1 + 1);

    int red = map(payload.toInt(), 0, 100, 0, PWM_VALUE);
    red = constrain(red, 0, PWM_VALUE);
    int green = map(payload.substring(c1 + 1, c2).toInt(), 0, 100, 0, PWM_VALUE);
    green = constrain(green, 0, PWM_VALUE);
    int blue = map(payload.substring(c2 + 1).toInt(), 0, 100, 0, PWM_VALUE);
    blue = constrain(blue, 0, PWM_VALUE);

    red = gamma_table[red];
    green = gamma_table[green];
    blue = gamma_table[blue];


    analogWrite(redPIN, red);
    analogWrite(greenPIN, green);
    analogWrite(bluePIN, blue);

    //   Serial1.println(red);
    //   Serial1.println(green);
    //   Serial1.println(blue);
  }
  else if (compareString(topic, "/openHAB/RGB_2/SW1")) {
    int value = map(payload.toInt(), 0, 100, 0, PWM_VALUE);
    value = constrain(value, 0, PWM_VALUE);
    value = gamma_table[value];
    analogWrite(w1PIN, value);
    //Serial1.println(value);
  }
  else if (compareString(topic, "/openHAB/RGB_2/SW2")) {
    int value = map(payload.toInt(), 0, 100, 0, PWM_VALUE);
    value = constrain(value, 0, PWM_VALUE);
    value = gamma_table[value];


    analogWrite(w2PIN, value);
    //Serial1.println(value);
  }
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect(MQTTCLIENTNAME,MQTTUSERNAME,MQTTPASSWORD)) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      //client.publish("outTopic", "hello world");
      // ... and resubscribe
      client.subscribe("/openHAB/RGB_2/Color");
      client.subscribe("/openHAB/RGB_2/SW1");
      client.subscribe("/openHAB/RGB_2/SW2");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
void loop() {

  if (!client.connected()) {
    reconnect();
  }
  client.loop();

//If you also want to publish something 
  //  long now = millis();
  //  if (now - lastMsg > 2000) {
  //    lastMsg = now;
  //    ++value;
  //    snprintf (msg, 75, "hello world #%ld", value);
  //    Serial.print("Publish message: ");
  //    Serial.println(msg);
  //    client.publish("outTopic", msg);
  //  }
}
